package cohy.correlator;
import 	 sep.operator.*;
import   jam.vector.*;
import   juice.vector.*;
import	 rsf.vector.*;
import   sep.io.ArgumentKeeper;

/**
 * Computes the correlation between all the traces of an input data set.
 * 2/(N(N-1)) \sum_i \sum_{j>i} ( \sum_k t_{ik} t_{jk} / 
 * \sqrt(\sum_k t_{ik}^2 \sum_k t_{jk}^2)). 
 * See Neiddel and Taner (Geophysics 36, No 3, Jun3 1971, P. 482-497).
 * The input and signal data can be 1-D, 2-D, or 3-D. The correlation is 
 * 1 if and only if all the traces are identical. 
 * Output is a RealVec.
 */
public class TraceCorrelator extends SepOperator {
  /**
   * Output space defaults to RealSpc.
   */
  public    TraceCorrelator(Space iputSpace ) { 
    super(iputSpace,RealSpc.getSpace()); 
  }
  protected TraceCorrelator(TraceCorrelator src) { super(src); }
  public jam.JamObject copy() { return new TraceCorrelator(this);}

  protected void apply(boolean add, Vector iput, Vector oput) {
    int dim  =  ((Rsf)iput).getDimension();
    switch (dim) {
    case 2: apply2d(add, iput, oput); break;
    case 3: apply3d(add, iput, oput); break;
    default:
      System.err.println("Error: dimension not implemented yet."); 
      break;
    }
  }

  void apply2d(boolean add, Vector iput, Vector oput) {
    float res  = 0.f; 
    float[][] data = ((Rsf2)   iput).d;
    int n1 = data.length;
    int n0 = data[0].length;
    for   (int i1 =    0; i1 < n1; i1++) {
    for   (int j1 = i1+1; j1 < n1; j1++) {
      res += Correlation.correlate(data[i1],data[j1]);
    }}
    res *= 2.f / (n1 * (n1-1)); 
    ((RealVec) oput).setValue(res);
  }
  void apply3d(boolean add, Vector iput, Vector oput) {
    float res  = 0.f; float corr = 0.f; 
    float nrmi = 0.f; float nrmj = 0.f;
    float[][][] data = ((Rsf3)   iput).d;
    int n2 = data.length;
    int n1 = data[0].length;
    int n0 = data[0][0].length;
    for   (int i2 =    0; i2 < n2; i2++) {
    for   (int i1 =    0; i1 < n1; i1++) {
    for   (int j2 = i2+1; j2 < n2; j2++) {
    for   (int j1 = i1+1; j1 < n1; j1++) {
      res += Correlation.correlate(data[i2][i1],data[j2][j1]);
    }}}}
    res *= 4.f / (n2 * (n2-1) * n1 * (n1-1)); 
    ((RealVec) oput).setValue(res);
  }
}



