package cohy.correlator;
import 	 sep.operator.*;
import   jam.vector.*;
import   juice.vector.*;
import	 rsf.vector.*;
import   sep.io.ArgumentKeeper;

/**
 * Computes the correlation between an input data set and a signal data set.
 * Each trace in the input data set is correlated with the corresponding 
 * trace in the signal data set. 
 * The input and signal data can be 1-D, 2-D, or 3-D but have to agree in 
 * shape (probably should agree in space but that is not checked).
 * Output is of the same shape as input and each trace is filled with the 
 * correlation coefficient (consequently constant). 
 * In the image function: input data reference can be identical to the 
 * constructor's signal reference.
 * @see: Correlator TraceCorrelator 
 */
public class TraceToTraceCorr extends SepOperator {
  Rsf     signal;
  /**
   * Input space equals the signal space. Output space defaults to RealSpc.
   */
  public TraceToTraceCorr(Rsf signal) {
    super(signal.getSpace(), signal.getSpace());
    this.signal = signal;
  }
  protected TraceToTraceCorr(TraceToTraceCorr src) {
    super(src);
    this.signal = (Rsf) src.signal.copy();
  }
  public jam.JamObject copy() { return new TraceToTraceCorr(this);}

  protected void apply(boolean add, Vector iput, Vector oput) {
    int dim  =  this.signal.getDimension();
    if ((dim != ((Rsf) iput).getDimension()) || 
	(dim != ((Rsf) oput).getDimension()) ) 
      System.err.println("Error: dimensional mismatch");
    switch (dim) {
    case 1: apply1d(add, iput, oput); break;
    case 2: apply2d(add, iput, oput); break;
    case 3: apply3d(add, iput, oput); break;
    default:
      System.err.println("Error: dimension not implemented yet."); 
      break;
    }
  }

  void apply1d(boolean add, Vector iput, Vector oput) {
    float[] idat = ((Rsf1)   iput).d;
    float[] sgnl = ((Rsf1) signal).d;
    ((Rsf) oput).fill(Correlation.correlate(idat, sgnl));
  }
  void apply2d(boolean add, Vector iput, Vector oput) {
    float res;
    float[][] idat = ((Rsf2)   iput).d;
    float[][] odat = ((Rsf2)   oput).d;
    float[][] sgnl = ((Rsf2) signal).d;
    int n1 = idat.length;
    int n0 = idat[0].length;
    for   (int i1 =    0; i1 < n1; i1++) {
      res = Correlation.correlate(idat[i1], sgnl[i1]);
      for (int i0 =    0; i0 < n0; i0++) {
	odat[i1][i0] = res; 
      }
    }
  }

  void apply3d(boolean add, Vector iput, Vector oput) {
    float res;
    float[][][] odat = ((Rsf3)   oput).d;
    float[][][] idat = ((Rsf3)   iput).d;
    float[][][] sgnl = ((Rsf3) signal).d;
    int n2 = idat.length;
    int n1 = idat[0].length;
    int n0 = idat[0][0].length;
    for   (int i2 = 0; i2 < n2; i2++) {
      for (int i1 = 0; i1 < n1; i1++) {
	res = Correlation.correlate(idat[i2][i1], sgnl[i2][i1]);
	for (int i0 = 0; i0 < n0; i0++) {
	  odat[i2][i1][i0] = res; 
	}
      }
    }
  }
}



