package   cohy.estDip;

import 	  sep.operator.*;
import    jam.vector.*;
import 	  rsf.vector.*;
import   cohy.gradient.*;
import   conv.convi.*;
import   sep.io.ArgumentKeeper;

/**
 * Computes the crossproduct 
 * <tt>|({\bf p \times \nabla) f| = |p| |\nabla f| \sin{\theta}</tt>. 
 * If <tt>f<tt> is a plane-wave and <tt>p</tt> its dip, the operator 
 * will return a null vector.<BR> 
 * In 2D the operator is simply the partial differential operator 
 * <tt>| p_y \partial_x + p_x \partial_y |</tt>, 
 * In 2D and 3D the operator returns a positive scalar for each input point.
 * The value is a measure of how the local point deviates from a plane-wave
 * of dip <tt>p</tt>.<BR> 
 * 
 * I may want to normalize the output by <tt>1 / |\nabla f|<tt>. 
 * 
 * <BR><!--#include file="./source.inc"--><BR>
 */
public class DipXGradLocalNorm extends SepOperator{
  DipEstimator dipy; // could create on the fly: clutter vs efficiency
  Vector       dips; // could create on the fly: clutter vs efficient?

  public DipXGradLocalNorm(RsfSpace iputSpace) {
    this(iputSpace, DipXGradLocalNorm.getOutputRsfSpace(iputSpace));
  }
  public DipXGradLocalNorm(RsfSpace iputSpace, RsfSpace oputSpace) {
    super(iputSpace, oputSpace);
    int dim   = iputSpace.getDimension();
    if (dim < 2 || dim > 3) System.err.println("Error: not implemented ..");
    this.dipy = new DipEstimator(iputSpace);
    this.dips = dipy.getRange().newMember();
  }
  protected DipXGradLocalNorm(DipXGradLocalNorm src) {
    super(src);
    dipy = new DipEstimator((RsfSpace) src.getDomain()); 
    dips = dipy.getRange().newMember(); // should this copy the values? 
  }
  public jam.JamObject copy() { return new DipXGradLocalNorm(this);}

  /**
   * This should be using the IcaiFactory class once that class implements
   * the OperatorFactory interface. Until then this class is a hack. 
   * I should probably call this createRange(). 
   */
  public static RsfSpace getOutputRsfSpace(RsfSpace iputSpace) {
    int n = iputSpace.getDimension();
    Axis[] oputAxes = new Axis[n];
    for (int i = 0 ; i < n ; i++) {
      oputAxes[i]          = new Axis(iputSpace.getAxes()[i]);
      oputAxes[i].nsamples =          iputSpace.getNSamples(i) -1;
    }
    return new RsfSpace(oputAxes); 
  }
  /**
   * I should call this createDomain(). More comments see getOutputRsfSpace.
   */
  public static RsfSpace getInputRsfSpace(RsfSpace oputSpace) {
    int n = oputSpace.getDimension();
    Axis[] iputAxes = new Axis[n];
    for (int i = 0 ; i < n ; i++) {
      iputAxes[i]          = new Axis(oputSpace.getAxes()[i]);
      iputAxes[i].nsamples =          oputSpace.getNSamples(i) +1;
    }
    return new RsfSpace(iputAxes); 
  }

  protected void apply(boolean add, Vector iput, Vector oput) {
    int dim  = ((Rsf) iput).getDimension();
    switch (dim) {
    case 2: apply2d(add, iput, oput); break;
    case 3: apply3d(add, iput, oput); break;
    default:
      System.err.println("Error: dimensions not implemented yet."); 
      break;
    }
  }

  void apply2d(boolean add, Vector iput, Vector oput) {
    dipy.image(iput, dips); 
    int[] idx = {DipEstimator.DX}; float dx = ((Rsf)dips).getDatum(idx);
    int[] idy = {DipEstimator.DY}; float dy = ((Rsf)dips).getDatum(idy);

    // create the convolution operator. 
    Rsf ff = PartDiff2d.getDerivativeDx();
    InternalConvolution diff = 
      new InternalConvolution( InternalConvolution.kInput , 
			       (RsfSpace) iput.getSpace(),
			       InternalConvolution.kFilter,
			       (RsfSpace) ff.getSpace());

    Rsf fd   = PartDiff2d.getDerivative(dy, -dx);
    diff.imageBinary(iput, fd, oput);
    rsf.operator.ElwAbs.elwAbs((Rsf) oput);
  }

  void apply3d(boolean add, Vector iput, Vector oput) {
    dipy.image(iput, dips); 
    int[] idx = {DipEstimator.DX}; float dx = ((Rsf)dips).getDatum(idx);
    int[] idy = {DipEstimator.DY}; float dy = ((Rsf)dips).getDatum(idy);
    int[] idz = {DipEstimator.DZ}; float dz = ((Rsf)dips).getDatum(idz);

    // create the convolution operator. 
    Rsf ff = PartDiff3d.getDerivativeDx();
    InternalConvolution diff = 
      new InternalConvolution( InternalConvolution.kInput , 
			       (RsfSpace) iput.getSpace(),
			       InternalConvolution.kFilter,
			       (RsfSpace) ff.getSpace());

    // I think I can avoid the dummy and cut verbosity by creating a ChainOp
    Rsf dummy = (Rsf) oput.getSpace().newMember();
    Rsf fd; 
    fd   = PartDiff3d.getDerivative(0.f, -dz,  dy);
    diff.imageBinary(iput, fd, dummy);
    rsf.operator.ElwSqr.elwSqr(dummy);
    oput.add(dummy);
    fd   = PartDiff3d.getDerivative( dz, 0.f, -dx);
    diff.imageBinary(iput, fd, dummy);
    rsf.operator.ElwSqr.elwSqr(dummy);
    oput.add(dummy); 
    fd   = PartDiff3d.getDerivative(-dy,  dx, 0.f);
    diff.imageBinary(iput, fd, dummy);
    rsf.operator.ElwSqr.elwSqr(dummy);
    oput.add(dummy); 
    rsf.operator.ElwSqrt.elwSqrt((Rsf) oput);
  }
}


