package   cohy.estDip;

import 	  sep.operator.*;
import    jam.vector.*;
import 	  rsf.vector.*;
import   cohy.gradient.*;
import   conv.convi.*;
import   sep.io.ArgumentKeeper;

/**
 * Computes the dotproducts of various partial derivative fields of the 
 * input. If <TT>f(x,y)<TT> is the input scalar field (Rsf), than this class 
 * computes the finite difference approximation of the partial derivatives
 * <TT>f_x, f_y</TT> and returns the dotproducts 
 * <PRE><f_x,f_x>,<f_x,f_y>,<f_y,f_y><\PRE>
 * In the case of input <TT>f(x,y,z)</TT> (3 dimensions) it returns 
 * <PRE><f_x,f_x>,<f_x,f_z>,<f_y,f_y>,<f_y,f_z>,<f_z,f_z><\PRE>. 
 * (The dotproducts are ordered alphabetically. I do not compute all 
 * derivatives since the dip estimation scheme assumes that plane-wave 
 * component is not vertical. Consequently, I need only these derivatives).
 * The class is needed for a dip estimation scheme. 
 * 
 * <BR><!--#include file="./source.inc"--><BR>
 */
public class PartDiffDots extends SepOperator {
  /** 2D indices that identify dotproducts in output array.*/
  public final static int XX2 = 0; 
  public final static int XY2 = 1;
  public final static int YY2 = 2;   
  /** 3D indices that identify dotproducts in output array.*/ 
  public final static int XX3 = 0; 
  public final static int XY3 = 1;
  public final static int XZ3 = 2;
  public final static int YY3 = 3;   
  public final static int YZ3 = 4; 
  public final static int ZZ3 = 5; 

  public PartDiffDots(RsfSpace iputSpace) {
    this(iputSpace, PartDiffDots.getOutputRsfSpace(iputSpace)); 
  }
  public PartDiffDots(RsfSpace iputSpace, RsfSpace oputSpace) {
    super(iputSpace, oputSpace);
    int dim = iputSpace.getDimension();
    if (dim < 2 || dim > 3) System.err.println("Error: not implemented ..");
  }
  public static RsfSpace getOutputRsfSpace(RsfSpace iputSpace) {
    if (iputSpace.getDimension() == 2) return new RsfSpace(3); 
    if (iputSpace.getDimension() == 3) return new RsfSpace(6);
    System.err.println("Error: dimension is not implemented yet");
    return new RsfSpace(0); 
  }
  protected PartDiffDots(PartDiffDots src) {
    super(src);
  }
  public jam.JamObject copy() { return new PartDiffDots(this);}

  protected void apply(boolean add, Vector iput, Vector oput) {
    // Should probably check iput isMember of domain.
    int dim  = ((Rsf) iput).getDimension();
    switch (dim) {
    case 2: apply2d(add, iput, oput); break;
    case 3: apply3d(add, iput, oput); break;
    default:
      System.err.println("Error: dimensions not implemented yet."); 
      break;
    }
  }

  void apply2d(boolean add, Vector iput, Vector oput) {
    // create the convolution operator. 
    Rsf ff = PartDiff2d.getDerivativeDx();
    InternalConvolution diff = 
      new InternalConvolution( InternalConvolution.kInput , 
			       (RsfSpace) iput.getSpace(),
			       InternalConvolution.kFilter,
			       (RsfSpace) ff.getSpace());

    // compute the partial derivatives 
    Rsf fx = PartDiff2d.getDerivativeDx();
    Rsf dx = (Rsf) diff.imageBinary(iput, fx);
    Rsf fy = PartDiff2d.getDerivativeDt();
    Rsf dy = (Rsf) diff.imageBinary(iput, fy);

    // compute the dotproducts 
    int[] xx = {XX2}; ((Rsf)oput).setDatum(xx,dx.dot(dx)); // f_x f_x
    int[] xy = {XY2}; ((Rsf)oput).setDatum(xy,dx.dot(dy)); // f_x f_y
    int[] yy = {YY2}; ((Rsf)oput).setDatum(yy,dy.dot(dy)); // f_y f_y
  }

  void apply3d(boolean add, Vector iput, Vector oput) {
    // create the convolution operator. 
    Rsf ff = PartDiff3d.getDerivativeDx();
    InternalConvolution diff = 
      new InternalConvolution( InternalConvolution.kInput , 
			       (RsfSpace) iput.getSpace(),
			       InternalConvolution.kFilter,
			       (RsfSpace) ff.getSpace());

    // compute the partial derivatives 
    Rsf fx = PartDiff3d.getDerivativeDx();
    Rsf dx = (Rsf) diff.imageBinary(iput, fx);
    Rsf fy = PartDiff3d.getDerivativeDy();
    Rsf dy = (Rsf) diff.imageBinary(iput, fy);
    Rsf fz = PartDiff3d.getDerivativeDz();
    Rsf dz = (Rsf) diff.imageBinary(iput, fz);

    // compute the dotproducts 
    int[] xx = {XX3}; ((Rsf)oput).setDatum(xx,dx.dot(dx)); // f_x f_x
    int[] xy = {XY3}; ((Rsf)oput).setDatum(xy,dx.dot(dy)); // f_x f_y
    int[] xz = {XZ3}; ((Rsf)oput).setDatum(xz,dx.dot(dz)); // f_x f_z
    int[] yy = {YY3}; ((Rsf)oput).setDatum(yy,dy.dot(dy)); // f_y f_y
    int[] yz = {YZ3}; ((Rsf)oput).setDatum(yz,dy.dot(dz)); // f_y f_z
    int[] zz = {ZZ3}; ((Rsf)oput).setDatum(zz,dz.dot(dz)); // f_z f_z
  }
}


